@@ -9,7 +9,7 @@ use std::io;
 use std::io::prelude::*;
 
 use crate::system_info::SystemInfo;
-use breakpad_symbols::FrameSymbolizer;
+use crate::FrameSymbolizer;
 use chrono::prelude::*;
 use minidump::system_info::Cpu;
 use minidump::*;
@@ -119,6 +119,8 @@ pub struct CallStack {
     pub frames: Vec<StackFrame>,
     /// Information about this `CallStack`.
     pub info: CallStackInfo,
+    /// The name of the thread, if known.
+    pub thread_name: Option<String>,
 }
 
 /// The state of a process as recorded by a `Minidump`.
@@ -157,6 +159,9 @@ pub struct ProcessState {
     // thread_memory_regions
     /// Information about the system on which the minidump was written.
     pub system_info: SystemInfo,
+    /// Linux Standard Base Info
+    pub linux_standard_base: Option<MinidumpLinuxLsbRelease>,
+    pub mac_crash_info: Option<Vec<RawMacCrashInfo>>,
     /// The modules that were loaded into the process represented by the
     /// `ProcessState`.
     pub modules: MinidumpModuleList,
@@ -164,6 +169,8 @@ pub struct ProcessState {
     // modules_without_symbols
     // modules_with_corrupt_symbols
     // exploitability
+    pub unknown_streams: Vec<MinidumpUnknownStream>,
+    pub unimplemented_streams: Vec<MinidumpUnimplementedStream>,
 }
 
 impl FrameTrust {
@@ -297,6 +304,7 @@ impl CallStack {
         CallStack {
             info,
             frames: vec![],
+            thread_name: None,
         }
     }
 
@@ -329,7 +337,7 @@ impl CallStack {
                         write!(
                             f,
                             " [{} : {} + {:#x}]",
-                            basename(&source_file),
+                            basename(source_file),
                             source_line,
                             addr - source_line_base
                         )?;
@@ -385,6 +393,13 @@ impl ProcessState {
                 ""
             }
         )?;
+        if let Some(ref lsb) = self.linux_standard_base {
+            writeln!(
+                f,
+                "Linux {} {} - {} ({})",
+                lsb.id, lsb.release, lsb.codename, lsb.description
+            )?;
+        }
         writeln!(f)?;
 
         if let (&Some(ref reason), &Some(ref address)) = (&self.crash_reason, &self.crash_address) {
@@ -401,6 +416,38 @@ Crash address: {:#x}
         if let Some(ref assertion) = self.assertion {
             writeln!(f, "Assertion: {}", assertion)?;
         }
+        if let Some(ref info) = self.mac_crash_info {
+            writeln!(f, "Mac Crash Info:")?;
+            for (idx, record) in info.iter().enumerate() {
+                writeln!(f, "  Record {}", idx)?;
+                if let Some(val) = record.thread() {
+                    writeln!(f, "    thread: 0x{}", val)?;
+                }
+                if let Some(val) = record.dialog_mode() {
+                    writeln!(f, "    dialog mode: 0x{}", val)?;
+                }
+                if let Some(val) = record.abort_cause() {
+                    writeln!(f, "    abort_cause: 0x{}", val)?;
+                }
+
+                if let Some(val) = record.module_path() {
+                    writeln!(f, "    module: {}", val)?;
+                }
+                if let Some(val) = record.message() {
+                    writeln!(f, "    message: {}", val)?;
+                }
+                if let Some(val) = record.signature_string() {
+                    writeln!(f, "    signature string: {}", val)?;
+                }
+                if let Some(val) = record.backtrace() {
+                    writeln!(f, "    backtrace: {}", val)?;
+                }
+                if let Some(val) = record.message2() {
+                    writeln!(f, "    message2: {}", val)?;
+                }
+            }
+            writeln!(f)?;
+        }
         if let Some(ref time) = self.process_create_time {
             let uptime = self.time - *time;
             writeln!(f, "Process uptime: {} seconds", uptime.num_seconds())?;
@@ -410,17 +457,19 @@ Crash address: {:#x}
         writeln!(f)?;
 
         if let Some(requesting_thread) = self.requesting_thread {
+            let stack = &self.threads[requesting_thread];
             writeln!(
                 f,
-                "Thread {} ({})",
+                "Thread {} {} ({})",
                 requesting_thread,
+                stack.thread_name.as_deref().unwrap_or(""),
                 if self.crashed() {
                     "crashed"
                 } else {
                     "requested dump, did not crash"
                 }
             )?;
-            self.threads[requesting_thread].print(f)?;
+            stack.print(f)?;
             writeln!(f)?;
         }
         for (i, stack) in self.threads.iter().enumerate() {
@@ -431,7 +480,12 @@ Crash address: {:#x}
             if stack.info == CallStackInfo::DumpThreadSkipped {
                 continue;
             }
-            writeln!(f, "Thread {}", i)?;
+            writeln!(
+                f,
+                "Thread {} {}",
+                i,
+                stack.thread_name.as_deref().unwrap_or("")
+            )?;
             stack.print(f)?;
         }
         write!(
@@ -471,6 +525,39 @@ Unloaded modules:
                 basename(&module.code_file()),
             )?;
         }
+        if !self.unimplemented_streams.is_empty() {
+            write!(
+                f,
+                "
+Unimplemented streams encountered:
+"
+            )?;
+            for stream in &self.unimplemented_streams {
+                writeln!(
+                    f,
+                    "Stream 0x{:08x} {:?} ({}) @ 0x{:08x}",
+                    stream.stream_type as u32,
+                    stream.stream_type,
+                    stream.vendor,
+                    stream.location.rva,
+                )?;
+            }
+        }
+        if !self.unknown_streams.is_empty() {
+            write!(
+                f,
+                "
+Unknown streams encountered:
+"
+            )?;
+            for stream in &self.unknown_streams {
+                writeln!(
+                    f,
+                    "Stream 0x{:08x} ({}) @ 0x{:08x}",
+                    stream.stream_type, stream.vendor, stream.location.rva,
+                )?;
+            }
+        }
         Ok(())
     }
 
@@ -505,15 +592,29 @@ Unloaded modules:
                 "crashing_thread": self.requesting_thread,
                 "assertion": self.assertion,
             },
+            // optional
+            "lsb_release": self.linux_standard_base.as_ref().map(|lsb| json!({
+                "id": lsb.id,
+                "release": lsb.release,
+                "codename": lsb.codename,
+                "description": lsb.description,
+            })),
+            // optional
+            "mac_crash_info": self.mac_crash_info.as_ref().map(|info| json!({
+                "num_records": info.len(),
+                // All of these fields are optional
+                "records": info.iter().map(|record| json!({
+                    "thread": record.thread().copied().map(json_hex),
+                    "dialog_mode": record.dialog_mode().copied().map(json_hex),
+                    "abort_cause": record.abort_cause().copied().map(json_hex),
 
-            // optional, Linux Standard Base information
-            // TODO: Issue #172
-            // "lsb_release": {
-            //   "id": <string>,
-            //   "release": <string>,
-            //   "codename": <string>,
-            //   "description": <string>
-            // },
+                    "module": record.module_path(),
+                    "message": record.message(),
+                    "signature_string": record.signature_string(),
+                    "backtrace": record.backtrace(),
+                    "message2": record.message2(),
+                })).collect::<Vec<_>>()
+            })),
 
             // the first module is always the main one
             "main_module": 0,
@@ -562,9 +663,8 @@ Unloaded modules:
                 // TODO: Issue #156
                 // optional
                 "last_error_value": null,
-                // TODO: Issue #173
                 // optional
-                "thread_name": null,
+                "thread_name": thread.thread_name,
                 "frames": thread.frames.iter().enumerate().map(|(idx, frame)| json!({
                     "frame": idx,
                     // optional
